--[[***************************************************************************
    * Name:      wxiscan.lua
    * Purpose:   Demo for implementing Lua addons for wxIScan.
    * Author:    Daniel Nell (daniel.nell@nellresearch.de)
    * Created:   2009-11-13
    * Copyright: Daniel Nell (www.nellresearch.de)
    * License:   GPL
    ***************************************************************************
--]]



-- ============================================================================
-- Redefine output function(s).
--
-- Replacement of the standard Lua print procedure that
-- sends the output to the LogMessage function which wraps
-- ::wxLogMessage(). So output goes to the standard log
-- target of the wxWidgets bases main program.
--
function print( ... )
    local args= {...}   -- Table for the function parameters.
    local text= ""      -- Accumulates the text for the output.

    -- Iterate on the function parameters and concatenate them.
    for index, value in ipairs( args ) do
        text= text .. tostring( value ) .. "\t"
    end

    -- Output via LogMessage (wxLuaScript::LogMessage).
    wxIScan.LogMessage( text )
end



-- ============================================================================
-- Usefull functions calling menu handlers on the C++ side.
--
-- Scan an image.
--
function wxIScan.ScanImage()
    return wxIScan.SendMenuEvent( ID_FILESCANIMAGE )
end


-- Print an image.
--
-- NOTE: This opens the printing dialog before printing.
--
function wxIScan.PrintImage()
    return wxIScan.SendMenuEvent( ID_FILEPRINT )
end


-- Do OCR and read out the text.
--
-- NOTE: The read out may be done asynchronously if
--       bAsync is set to 'true'.
--
function wxIScan.DocumentReadText( bAsync )
    -- Handle default parameter(s).
    if bAsync == nil then
        bAsync= true
    end

    -- Read out.
    if bAsync then
        return wxIScan.SendMenuEvent( ID_DOCUMENTREADTEXT )
    else
        return wxIScan.SendMenuEvent( ID_DOCUMENTREADTEXTSYNC )
    end
end


-- Turn a multiple image document one page back.
--
function wxIScan.DocumentPreviousPage()
    return wxIScan.SendMenuEvent( ID_DOCUMENTPREVIOUSPAGE )
end


-- Turn a multiple image document one page forward.
--
function wxIScan.DocumentNextPage()
    return wxIScan.SendMenuEvent( ID_DOCUMENTNEXTPAGE )
end


-- Stop an asynchronous running sub process.
--
function wxIScan.CmdStop()
    return wxIScan.SendMenuEvent( ID_CMDSTOP )
end


-- Add the current loaded image as a new page to the
-- current PDF document.
--
function wxIScan.PdfAddPage()
    return wxIScan.SendMenuEvent( ID_PDFADDPAGE )
end


-- ============================================================================
-- Main chunks
-- (referred by the "Script" section in the config file).
--
-- Scan an image and send it to the printer.
--
function CarbonCopy()
    return wxIScan.ScanImage() and wxIScan.PrintImage()
end


-- Scan an image and read out its text (using OCR).
--
function ScanAndRead()
    return wxIScan.ScanImage() and wxIScan.CmdStop() and wxIScan.DocumentReadText()
end


-- Turn pages of a multiple image document (TIFF or PDF)
-- and read out the document page by page (using OCR and
-- TTS).
--
function TurnPagesAndRead()
    if wxIScan.CmdStop() and wxIScan.DocumentReadText( false ) then
        while wxIScan.DocumentNextPage() and wxIScan.CmdStop() and wxIScan.DocumentReadText( false ) do
        end
    end
end


-- Do OCR via cuneiform, export it as an RTF document,
-- and open OpenOffice.org/LibreOffice as RTF editor --
-- or any RTF viewer given.
--
function Ocr2RtfViewer( Language, Cmd )
    -- Handle default parameter(s).
    if Language == nil then
        Language= "en"
    end
    if Cmd == nil then
        Cmd= "soffice" -- Set OpenOffice.org/LibreOffice as default.
    end

    -- Create a temporary file(name).
    local TempFileName= os.tmpname()

    -- Create temporary filename(s) from basename.
    local TempFileNameText= TempFileName .. ".rtf"

    -- OCR commands for various languages.
    local OcrCmd=
    {
        ["de"]= "cuneiform -l ger -f rtf -o " .. TempFileNameText .. " " .. TempFileName;
        ["en"]= "cuneiform -l eng -f rtf -o " .. TempFileNameText .. " " .. TempFileName
    }

    -- Convert the image to a monochrome image.
    wxIScan.Image2Mono()

    -- Save the image to the given name (as tiff file).
    wxIScan.SaveImage( TempFileName, "image/tiff" )

    -- Execute the OCR software and create a rich text file.
    wxIScan.Execute( OcrCmd[Language] )

    -- Open the rtf viewer for editing.
    os.execute( Cmd .. " " .. TempFileNameText )

    -- Remove temporary files.
    os.remove( TempFileName )
    os.remove( TempFileNameText )
end


-- Do OCR via cuneiform or tesseract, export it as an hOCR
-- document, turn it into a seekable PDF document using
-- hocr2pdf from ExcactImage, and open it using evince.
--
function Ocr2SeekablePdf( Language, Ocr )
    -- Handle default parameter(s).
    if Language == nil then
        Language= "en"
    end
    if Ocr == nil then
        Ocr= "cuneiform"
    end

    -- Create a temporary file(name).
    local TempFileName= os.tmpname()

    -- Create temporary filename(s) from basename.
    local TempFileNameText= TempFileName .. ".html"
    local TempFileNamePdf= TempFileName .. ".pdf"

    -- OCR commands for various languages.
    local OcrCmd=
    {
        ["cuneiform-de"]= "cuneiform -l ger -f hocr -o " .. TempFileNameText .. " " .. TempFileName;
        ["cuneiform-en"]= "cuneiform -l eng -f hocr -o " .. TempFileNameText .. " " .. TempFileName;
        ["tesseract-de"]= "tesseract " .. TempFileName .. " " .. TempFileName .. " -l deu hocr";
        ["tesseract-en"]= "tesseract " .. TempFileName .. " " .. TempFileName .. " -l eng hocr"
    }

    -- Convert the image to a monochrome image.
    wxIScan.Image2Mono()

    -- Save the image to the given name (as tiff file).
    wxIScan.SaveImage( TempFileName, "image/tiff" )

    -- Execute the OCR software and create a HOCR html/xhtml file.
    local Cmd= OcrCmd[Ocr .. "-" .. Language]
    print( Cmd )
    wxIScan.Execute( Cmd )

    -- Combine tiff image and hocr html-text into a seekable PDF file.
    Cmd= "hocr2pdf -i " .. TempFileName .. " -o " .. TempFileNamePdf .. " < " .. TempFileNameText
    print( Cmd )
    os.execute( Cmd )

    -- Open the PDF file.
    Cmd= "evince " .. TempFileNamePdf
    print( Cmd )
    os.execute( Cmd )

    -- Remove temporary files.
    os.remove( TempFileName )
    os.remove( TempFileNameText )
    os.remove( TempFileNamePdf )

    -- Remove temporary image directory created by cuneiform.
    if Ocr == "cuneiform" then
        Cmd= "rm -r " .. TempFileName .. "_files"
        print( Cmd )
        os.execute( "rm -r " .. TempFileName .. "_files" )
    end
end


-- Do OCR via cuneiform using the given language and
-- read it out using espeak, mbrola, aplay and the
-- given language.
--
function ReadOut( Language )
    -- Handle default parameter(s).
    if Language == nil then
        Language= "en"
    end

    -- Create a temporary file(name).
    local TempFileName= os.tmpname()

    -- Create temporary filename(s) from basename.
    local TempFileNameText= TempFileName .. ".txt"

    -- OCR commands for various languages.
    local OcrCmd=
    {
        ["de"]= "cuneiform -l ger -f text -o " .. TempFileNameText .. " " .. TempFileName;
        ["en"]= "cuneiform -l eng -f text -o " .. TempFileNameText .. " " .. TempFileName
    }

    -- Read out commands for various languages.
    local ReadOutCmd=
    {
        ["de"]= "sh -c 'espeak -p 75 -s 140 -v mb-de6 -f " .. TempFileNameText
                .. " | mbrola -e - - | aplay -r22050 -fS16'";
        ["en"]= "sh -c 'espeak -p 60 -s 160 -v mb-en1 -f " .. TempFileNameText
                .. " | mbrola -e - - | aplay -r22050 -fS16'"
    }

    -- Convert the image to a monochrome image.
    wxIScan.Image2Mono()

    -- Save the image to the given name (as TIFF file).
    wxIScan.SaveImage( TempFileName, "image/tiff" )

    -- Execute the OCR software and create a plain text file.
    wxIScan.Execute( OcrCmd[Language] )

    -- Read out the text file.
    wxIScan.Execute( ReadOutCmd[Language], true, TempFileNameText )

    -- Remove temporary files.
    os.remove( TempFileName )
end


-- 'Unpaper' (deskew) the program's image using unpaper
-- and the given (unpaper command line) options.
--
function Unpaper( Options )
    -- Handle default parameter(s).
    if Options == nil then
        Options= ""
    end

    -- Create temporary file(name)s.
    local TempFileName= os.tmpname()
    local TempFileNameOut= os.tmpname()

    -- Save input image.
    wxIScan.SaveImage( TempFileName, "image/pnm" )

    -- Execute unpaper program.
    local Cmd= "unpaper " .. Options .. " --overwrite " .. TempFileName .. " " .. TempFileNameOut
    wxIScan.Execute( Cmd )

    -- Load output image
    wxIScan.OpenImage( TempFileNameOut )

    -- Remove temporary files.
    os.remove( TempFileName )
    os.remove( TempFileNameOut )
end


-- Automatically enhence the program's image (a photo) using
-- aaphoto and the given (aaphoto command line) options.
--
function aaphoto( Options )
    -- Handle default parameter(s).
    if Options == nil then
        Options= ""
    end

    -- Create temporary file(name)s.
    local TempFileName= os.tmpname()
    local TempFileNameOut= TempFileName .. '.pnm'

    -- Save input image.
    wxIScan.SaveImage( TempFileNameOut, "image/pnm" )

    -- Execute unpaper program.
    local Cmd= "aaphoto " .. Options .. " --autoadjust --overwrite " .. TempFileNameOut
    wxIScan.Execute( Cmd )

    -- Load output image
    wxIScan.OpenImage( TempFileNameOut )

    -- Remove temporary files.
    os.remove( TempFileName )
    os.remove( TempFileNameOut )
end



-- ============================================================================
-- Main chunks
-- (referred by the "Script" section in the config file).
--
-- Common test function (test bed) for Lua bindings.
--
function Test()
    print( "LUA: " .. _VERSION )
    print( "LUA: Current date and time is " .. os.date( "%x %X", os.time() ) .. '.' )
    if os.getenv( "HOME" ) ~= nil then
        print( "LUA: Your home directory is '" .. os.getenv( "HOME" ) .. "'." )
    end
    print( "LUA: package.cpath == '" .. package.cpath .. "'" )
    print( "LUA: Scanner/Image resolution: " .. Resolution() .. " DPI" )
    print( "LUA: Pages in image file: " .. ImageCount() )
    print( "LUA: Image page: " .. Page() + 1 )
    print( "LUA: Pages to write to PDF file: " .. PdfPages() )
    print( "LUA: Current image processing profile: " .. ProfileIndex() )
    print( "LUA: Image file name: " .. ImageFileName() )
    print( "LUA: BitmapMimeType == '" .. BitmapMimeType() .. "'" )
    print( "LUA: PdfBitmapMimeType == '" .. PdfBitmapMimeType() .. "'" )
    print( "LUA: ScaleFactor == " .. ScaleFactor() )
    print( "LUA: ZoomFactor == " .. ZoomFactor() )
--    print( "LUA: AutoApplyProfile == " .. AutoApplyProfile() )
--    print( "LUA: AutoApplyProfileOnMultiPage == " .. AutoApplyProfileOnMultiPage() )
--    wxIScan.EnhenceColours( -10, 20 )
--    wxIScan.Image2Mono( 192 )
--    wxIScan.Image2Grey()
--    wxIScan.CropImage( 100, 200, 500, 800 )
--    wxIScan.RotateImage( -1.0 / 360 * ( 2 * math.pi ) )
--    wxIScan.RotateImage90( false )
--    wxIScan.EnhenceOpticalContrast()
--    wxIScan.ResizeImage( 200, 200 )
--    wxIScan.ResetFileName()
--    wxIScan.CopyText2Clipboard( "text from Lua in clipboard" )
end
