/***************************************************************
 * Name:      wxiscanluascript.h
 * Purpose:   Provides access to a lot of wxIScan functions
 *            to the Lua language.
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2012-02-03
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   GPL
 **************************************************************/

#ifndef WXISCANLUASCRIPT_H
#define WXISCANLUASCRIPT_H

// Private header files.
#if __WXLUASCRIPT__
#   include "wxluascript.h"
#endif // __WXLUASCRIPT__


//////////////////////////////////////////////////////////
// Predefined classes
//
#if __WXLUASCRIPT__
struct lua_State;
#endif // __WXLUASCRIPT__


#if __WXLUASCRIPT__
/////////////////////////////////////////////////////////////////////////////
//  Class wxIScanLuaScript
//
/// \brief An extended class to handle Lua scripting.
///
/// wxIScanLuaScript is a friend class to the frame window class
/// wxIScanFrame an is intended to control the frame window
/// remotely.
class wxIScanLuaScript : public wxLuaScript
{
  public:
    /// \brief Standard constructor.
    ///
    /// \param strScript            The Lua source code of the script to execute.
    ///                             Default is the empty string.
    /// \param poParent             a pointer to the parent event handler (mostly
    ///                             a window), may be NULL
    /// \param strDomain            ...
    ///                             Default is the empty string.
    /// \param pbParentState        a pointer to a boolean to get the error state,
    ///                             may be NULL
    wxIScanLuaScript( const wxString& strScript= wxEmptyString, wxEvtHandler *poParent= NULL,
                      const wxString& strDomain= wxEmptyString, bool *pbParentState= NULL );

  private:
    // Addtional Lua functions.
    /// \brief Do OCR on the loaded/scanned image and return the text (to Lua).
    ///
    /// \param L      a pointer to the Lua context.
    static int OCR( lua_State* L );

    /// \brief Open an image file in Lua using a given name.
    ///
    /// \param L      a pointer to the Lua context.
    static int OpenImage( lua_State* L );

    /// \brief Save an image file in Lua using a given name.
    ///
    /// \param L      a pointer to the Lua context.
    static int SaveImage( lua_State* L );

    /// \brief Copy a given text to the clipboard in Lua.
    ///
    /// \param L      a pointer to the Lua context.
    static int CopyText2Clipboard( lua_State* L );

    /// \brief Replace the program's image by its monochrome version.
    ///
    /// \param L      a pointer to the Lua context.
    static int Image2Mono( lua_State* L );

    /// \brief Replace the program's image by its greyscale version.
    ///
    /// \param L      a pointer to the Lua context.
    static int Image2Grey( lua_State* L );

    /// \brief Replace the program's image by its cropped version.
    ///
    /// \param L      a pointer to the Lua context.
    static int CropImage( lua_State* L );

    /// \brief Replace the program's image by a new version that is colour enhenced.
    ///
    /// \param L      a pointer to the Lua context.
    static int EnhenceColours( lua_State* L );

    /// \brief Replace white colour by yellow.
    ///
    /// \param L      a pointer to the Lua context.
    static int EnhenceOpticalContrast( lua_State* L );

    /// \brief (Fine) Rotate an image by a given angle.
    ///
    /// \param L      a pointer to the Lua context.
    static int RotateImage( lua_State* L );

    /// \brief Rotate an image by a given angle by 90 degrees.
    ///
    /// \param L      a pointer to the Lua context.
    static int RotateImage90( lua_State* L );

    /// \brief Resize an image to the given size.
    ///
    /// \param L      a pointer to the Lua context.
    static int ResizeImage( lua_State* L );

    /// \brief Fork an external process.
    ///
    /// \param L      a pointer to the Lua context.
    static int Execute( lua_State* L );

    /// \brief Set the filename of the image to "unknown.XXX".
    ///
    /// \param L      a pointer to the Lua context.
    static int ResetFileName( lua_State* L );
};
#endif // __WXLUASCRIPT__

#endif // WXISCANHOCR2PDF_H
